#include "librework_grp.h"

static FILE * grent_file = NULL;
static QMutex mutex;

enum nss_status _nss_librework_getgrnam_r( const char *name, struct group *g, char *buffer, size_t buflen, int * errnop)
{
	enum nss_status ret;
	mutex.lock();
	ret = _getgrnam_r(name, g, buffer, buflen, errnop);
	mutex.unlock();
	return ret;
}

enum nss_status _getgrnam_r( const char *name, struct group *g, char *buffer, size_t buflen, int * errnop)
{
	Logger::logInfo( "_nss_librework_getgrnam_r" );
	struct group *conf;
  
	if ((conf = LibreworkFile::readGroupConf( name )) == NULL)
	{
		Logger::logInfo( "_nss_librework_getgrnam_r: NSS_STATUS_NOTFOUND" );
		return NSS_STATUS_NOTFOUND;
	}

	*g = *conf;

	strcpy(g->gr_passwd, "x");

	Logger::logInfo( QString("_nss_librework_getgrnam_r: gr_name: %1, gr_passwd: %2").arg(g->gr_name).arg(g->gr_passwd));

	return NSS_STATUS_SUCCESS;
}

enum nss_status _nss_librework_getgrgid_r(gid_t gid, struct group *g, char *buffer, size_t buflen, int * errnop)
{
	enum nss_status ret;
	mutex.lock();
	ret = _getgrgid_r(gid, g, buffer, buflen, errnop);
	mutex.unlock();
	return ret;
}

enum nss_status _getgrgid_r(gid_t gid, struct group *g, char *buffer, size_t buflen, int * errnop)
{
	Logger::logInfo( QString("_nss_librework_getgrgid_r gid: %1").arg(gid) );

	if ( gid == 0 )
	{
		Logger::logInfo( "_nss_librework_getgrgid_r: NULL gid: NSS_STATUS_NOTFOUND" );
		return NSS_STATUS_NOTFOUND;
	}

	QString nameString = LibreworkFile::getGroupName( gid );

	if ( nameString.isEmpty() )
	{
		Logger::logInfo( "_nss_librework_getgrgid_r: Name not found: NSS_STATUS_NOTFOUND" );
		return NSS_STATUS_NOTFOUND;
	}

	struct group *conf;
	char * name = nameString.toUtf8().data();
  
	if ((conf = LibreworkFile::readGroupConf( name )) == NULL)
	{
		Logger::logInfo( "_nss_librework_getgrgid_r: NSS_STATUS_NOTFOUND" );
		return NSS_STATUS_NOTFOUND;
	}

	*g = *conf;

	strcpy(g->gr_passwd, "x");

	Logger::logInfo( QString("_nss_librework_getgrgid_r: gr_name: %1, gr_passwd: %2").arg(g->gr_name).arg(g->gr_passwd));
	return NSS_STATUS_SUCCESS;
}

enum nss_status _nss_librework_getgrent_r(struct group *g, char *buffer, size_t buflen, int *errnop)
{
	enum nss_status ret;
	mutex.lock();
	ret = _getgrent_r(g, buffer, buflen, errnop);
	mutex.unlock();
	return ret;
}

enum nss_status _getgrent_r(struct group *g, char *buffer, size_t buflen, int *errnop)
{
	Logger::logInfo( "_nss_librework_getgrent_r" );

	enum nss_status ret = NSS_STATUS_SUCCESS;

	if (grent_file == NULL)
	{
		Logger::logInfo("grent_file == NULL, going to setgrent\n");
		ret = _setgrent(0);
	}

	if (ret == NSS_STATUS_SUCCESS)
	{
		if (fgetgrent_r(grent_file, g, buffer, buflen, &g) == 0)
		{
			Logger::logInfo(QString("Returning group %1:%2").arg( g->gr_gid ).arg( g->gr_name ) );
		} else {
			if (errno == ENOENT)
			{
				errno = 0;
			}
			*errnop = errno;
			if ( *errnop == ERANGE )
			{
				ret = NSS_STATUS_TRYAGAIN;
			} else {
				ret = NSS_STATUS_NOTFOUND;
			}
		}
	}

	return ret;
}

enum nss_status _nss_librework_setgrent(int stayopen)
{
	Logger::logInfo( "_nss_librework_setgrent" );
	enum nss_status ret;
	mutex.lock();
	ret = _setgrent(stayopen);
	mutex.unlock();
	return ret;
}

enum nss_status _setgrent(int stayopen)
{
	Logger::logInfo( QString( "_nss_librework_setgrent : opening %1" ).arg( LIBREWORK_GROUP_FILE ) );
	grent_file = fopen(LIBREWORK_GROUP_FILE, "r");

	if (grent_file)
	{
		return NSS_STATUS_SUCCESS;
	} else {
		return NSS_STATUS_UNAVAIL;
	}
}

enum nss_status _endgrent(void)
{
	Logger::logInfo("Closing librework nss group file");
	if (grent_file)
	{
		fclose(grent_file);
		grent_file = NULL;
	}
	return NSS_STATUS_SUCCESS;
}

enum nss_status _nss_librework_endgrent(void)
{
	enum nss_status ret;
	mutex.lock();
	ret = _endgrent();
	mutex.unlock();
	return ret;
}
