#include "librework_pwd.h"

static FILE * pwent_file = NULL;
static QMutex mutex;

enum nss_status _nss_librework_getpwnam_r( const char *name, struct passwd *p, char *buffer, size_t buflen, int *errnop)
{
	enum nss_status ret;
	mutex.lock();
	ret = _getpwnam_r(name, p, buffer, buflen, errnop);
	mutex.unlock();
	return ret;
}

enum nss_status _getpwnam_r( const char *name, struct passwd *p, char *buffer, size_t buflen, int *errnop)
{
	Logger::logInfo( "_nss_librework_getpwnam_r" );
	struct passwd *conf;

	QString nameString = name;
  
	if ((conf = LibreworkFile::readPasswdConf( nameString )) == NULL)
	{
		Logger::logInfo( "_nss_librework_getpwnam_r: NSS_STATUS_NOTFOUND" );
		return NSS_STATUS_NOTFOUND;
	}

	*p = *conf;

	strcpy(p->pw_passwd, "x");

	Logger::logInfo( QString("_nss_librework_getpwnam_r: pw_name: %1, pw_passwd: %2").arg(p->pw_name).arg(p->pw_passwd));

	return NSS_STATUS_SUCCESS;
}

enum nss_status _nss_librework_getpwuid_r(uid_t uid, struct passwd *p, char *buffer, size_t buflen, int *errnop)
{
	enum nss_status ret;
	mutex.lock();
	ret = _getpwuid_r(uid, p, buffer, buflen, errnop);
	mutex.unlock();
	return ret;
}

enum nss_status _getpwuid_r(uid_t uid, struct passwd *p, char *buffer, size_t buflen, int *errnop)
{
	Logger::logInfo( QString("_nss_librework_getpwuid_r uid: %1").arg(uid) );

	if ( uid == 0 )
	{
		Logger::logInfo( "_nss_librework_getpwuid_r: NULL uid: NSS_STATUS_NOTFOUND" );
		return NSS_STATUS_NOTFOUND;
	}

	QString nameString = LibreworkFile::getName( uid );

	if ( nameString.isEmpty() )
	{
		Logger::logInfo( "_nss_librework_getpwuid_r: Name not found: NSS_STATUS_NOTFOUND" );
		return NSS_STATUS_NOTFOUND;
	}

	struct passwd *conf;
	char * name = nameString.toUtf8().data();
  
	if ((conf = LibreworkFile::readPasswdConf( nameString )) == NULL)
	{
		Logger::logInfo( "_nss_librework_getpwuid_r: NSS_STATUS_NOTFOUND" );
		return NSS_STATUS_NOTFOUND;
	}

	*p = *conf;

	//strcpy(p->pw_name, name);
	strcpy(p->pw_passwd, "x");

	Logger::logInfo( QString("_nss_librework_getpwuid_r: pw_name: %1, pw_passwd: %2").arg(p->pw_name).arg(p->pw_passwd));
	return NSS_STATUS_SUCCESS;
}

enum nss_status _nss_librework_getpwent_r(struct passwd *p, char *buffer, size_t buflen, int *errnop)
{
	enum nss_status ret;
	mutex.lock();
	ret = _getpwent_r(p, buffer, buflen, errnop);
	mutex.unlock();
	return ret;
}

enum nss_status _getpwent_r(struct passwd *p, char *buffer, size_t buflen, int *errnop)
{
	Logger::logInfo( "_nss_librework_getpwent_r" );

	enum nss_status ret = NSS_STATUS_SUCCESS;

	if (pwent_file == NULL)
	{
		Logger::logInfo("p_file == NULL, going to setpwent\n");
		ret = _setpwent(0);
	}

	if (ret == NSS_STATUS_SUCCESS)
	{
		if (fgetpwent_r(pwent_file, p, buffer, buflen, &p) == 0)
		{
			Logger::logInfo(QString("Returning user %1:%2").arg( p->pw_uid ).arg( p->pw_name ) );
		} else {
			if (errno == ENOENT)
			{
				errno = 0;
			}
			*errnop = errno;
			if ( *errnop == ERANGE )
			{
				ret = NSS_STATUS_TRYAGAIN;
			} else {
				ret = NSS_STATUS_NOTFOUND;
			}
		}
	}

	return ret;
}

enum nss_status _nss_librework_setpwent(int stayopen)
{
	Logger::logInfo( "_nss_librework_setpwent" );
	enum nss_status ret;
	mutex.lock();
	ret = _setpwent(stayopen);
	mutex.unlock();
	return ret;
}

enum nss_status _setpwent(int stayopen)
{

	Logger::logInfo( QString( "_nss_librework_setpwent : opening %1" ).arg( LIBREWORK_PASSWD_FILE ) );
	pwent_file = fopen(LIBREWORK_PASSWD_FILE, "r");

	if (pwent_file)
	{
		return NSS_STATUS_SUCCESS;
	} else {
		return NSS_STATUS_UNAVAIL;
	}
}

enum nss_status _endpwent(void)
{
	Logger::logInfo("Closing librework nss file");
	if (pwent_file)
	{
		fclose(pwent_file);
		pwent_file = NULL;
	}
	return NSS_STATUS_SUCCESS;
}

enum nss_status _nss_librework_endpwent(void)
{
	enum nss_status ret;
	mutex.lock();
	ret = _endpwent();
	mutex.unlock();
	return ret;
}
