#include "librework_shadow.h"

static FILE * spent_file = NULL;
static QMutex mutex;

enum nss_status _nss_librework_getspnam_r( const char *name, struct spwd *p, char *buffer, size_t buflen, int *errnop)
{
	enum nss_status ret;
	mutex.lock();
	ret = _getspnam_r(name, p, buffer, buflen, errnop);
	mutex.unlock();
	return ret;
}

enum nss_status _getspnam_r( const char *name, struct spwd *p, char *buffer, size_t buflen, int *errnop)
{
	Logger::logInfo( "_nss_librework_getspnam_r" );
	struct spwd *conf;
  
	if ((conf = LibreworkFile::readShadowConf( name )) == NULL)
	{
		Logger::logInfo( "_nss_librework_getspnam_r: NSS_STATUS_NOTFOUND" );
		return NSS_STATUS_NOTFOUND;
	}

	*p = *conf;

	strcpy(p->sp_pwdp, "x");

	Logger::logInfo( QString("_nss_librework_getspnam_r: sp_namp: %1, sp_pwdp: %2").arg(p->sp_namp).arg(p->sp_pwdp));

	return NSS_STATUS_SUCCESS;
}

enum nss_status _nss_librework_getspent_r(struct spwd *p, char *buffer, size_t buflen, int *errnop)
{
	enum nss_status ret;
	mutex.lock();
	ret = _getspent_r(p, buffer, buflen, errnop);
	mutex.unlock();
	return ret;
}

enum nss_status _getspent_r(struct spwd *p, char *buffer, size_t buflen, int *errnop)
{
	Logger::logInfo( "_nss_librework_getspent_r" );

	enum nss_status ret = NSS_STATUS_SUCCESS;

	if (spent_file == NULL)
	{
		Logger::logInfo("p_file == NULL, going to setspent\n");
		ret = _setspent(0);
	}

	if (ret == NSS_STATUS_SUCCESS)
	{
		if (fgetspent_r(spent_file, p, buffer, buflen, &p) == 0)
		{
			Logger::logInfo(QString("Returning user shadow %1").arg( p->sp_namp ) );
		} else {
			if (errno == ENOENT)
			{
				errno = 0;
			}
			*errnop = errno;
			if ( *errnop == ERANGE )
			{
				ret = NSS_STATUS_TRYAGAIN;
			} else {
				ret = NSS_STATUS_NOTFOUND;
			}
		}
	}

	return ret;
}

enum nss_status _nss_librework_setspent(int stayopen)
{
	Logger::logInfo( "_nss_librework_setspent" );
	enum nss_status ret;
	mutex.lock();
	ret = _setspent(stayopen);
	mutex.unlock();
	return ret;
}

enum nss_status _setspent(int stayopen)
{

	Logger::logInfo( QString( "_nss_librework_setspent : opening %1" ).arg( LIBREWORK_SHADOW_FILE ) );
	spent_file = fopen(LIBREWORK_SHADOW_FILE, "r");

	if (spent_file)
	{
		return NSS_STATUS_SUCCESS;
	} else {
		return NSS_STATUS_UNAVAIL;
	}
}

enum nss_status _endspent(void)
{
	Logger::logInfo("Closing librework nss shadow file");
	if (spent_file)
	{
		fclose(spent_file);
		spent_file = NULL;
	}
	return NSS_STATUS_SUCCESS;
}

enum nss_status _nss_librework_endspent(void)
{
	enum nss_status ret;
	mutex.lock();
	ret = _endspent();
	mutex.unlock();
	return ret;
}
