#include "libreworkclient.h"

struct passwd * LibreworkClient::readConf( const char * name )
{
	QSettings settings( LIBREWORK_DESKTOP_CONFFILE, QSettings::IniFormat );
	QString address = settings.value("Server/LibreworkServerAddress").toString();
	int port = settings.value("Server/LibreworkServerPort").toInt();
	if ( address.isEmpty() || port <= 0 )
	{
		Logger::logError( QString("Cannot read Librework Server address or port number from config file %1. Exiting").arg(LIBREWORK_DESKTOP_CONFFILE) );
		return NULL;
	}
	
	SOCKET socket = LWSocket::createRemoteSocket( address, port );
	if ( socket < 0 )
	{
		Logger::logError( QString("Something happened creating client socket. Exiting") );
		return NULL;
	}

	QString password;

	QString line = QString("login?%1&&%2").arg(name).arg(password); 
	if ( ! LWSocket::sendMessage( socket, line ) )
	{
		Logger::logError( QString("Cannot authenticate against Librework Server. Exiting") );
		return NULL;
	}

	line = QString("get_user_uid?%1").arg(name); 

	if ( ! LWSocket::sendMessage( socket, line ) )
	{
		Logger::logError( QString("Cannot send message to Librework Server. Exiting") );
		return NULL;
	}

	QString response = LWSocket::receiveMessage( socket );
	bool ok;
	long long uid = response.toLongLong( &ok );

	if ( ! ok || uid <= 0 )
	{
		Logger::logError( QString("Cannot get uid for user %1 from Librework Server. Exiting").arg(name) );
		return NULL;
	}

	QTemporaryFile file;
	QString fileName;
	if ( file.open() )
	{
		fileName = file.fileName();
		QByteArray out = QString("%1:x:%2:50::/home/%1:/bin/bash\n").arg(name).arg(uid).toUtf8();
		file.write( out );
		file.close();
	} else {
		Logger::logError( QString("Cannot write passwd temporary file. Exiting") );
		return NULL;
	}

	FILE *fd;
	struct passwd *conf;

	if ((fd = fopen(fileName.toUtf8().data(), "r")) == NULL )
	{
		return NULL;
	}
	conf = fgetpwent(fd);

	fclose(fd);

	file.remove();

	return conf;
}
