#include "libreworkcommon.h"

/* 
 * Allocate some space from the nss static buffer.  The buffer and buflen
 * are the pointers passed in by the C library to the _nss_ntdom_*
 * functions. 
 *
 *  Taken from glibc 
 */

char * LibreworkCommon::getStatic(char **buffer, size_t *buflen, int len)
{
	char *result;

	/* Error check.  We return false if things aren't set up right, or
         * there isn't enough buffer space left. */

	if ((buffer == NULL) || (buflen == NULL) || (*buflen < len)) {
		return NULL;
	}

	/* Return an index into the static buffer */

	result = *buffer;
	*buffer += len;
	*buflen -= len;

	return result;
}

struct passwd * LibreworkCommon::getPasswd(QString line, struct passwd * p)
{
	QStringList fields = line.split(':');

	if ( fields.count() == 7 )
	{
		//struct passwd * p = (struct passwd *) malloc( sizeof( struct passwd ) );

		bool ok;

		uid_t uid = fields[2].toUInt(&ok);
		if ( ok )
		{
			p->pw_uid = uid;
		} else {
			//free( p );
			return NULL;
		}

		gid_t gid = fields[3].toUInt(&ok);
		if ( ok )
		{
			p->pw_gid = gid;
		} else {
			//free( p );
			return NULL;
		}

		p->pw_name = (char *) malloc( fields[0].size() + 1 );
		strcpy(p->pw_name, fields[0].toUtf8().data());

		p->pw_passwd = (char *) malloc( fields[1].size() + 1 );
		strcpy(p->pw_passwd, fields[1].toUtf8().data());

		p->pw_gecos = (char *) malloc( fields[4].size() + 1 );
		strcpy(p->pw_gecos, fields[4].toUtf8().data());

		p->pw_dir = (char *) malloc( fields[5].size() + 1 );
		strcpy(p->pw_dir, fields[5].toUtf8().data());

		p->pw_shell = (char *) malloc( fields[6].size() + 1 );
		strcpy(p->pw_shell, fields[6].toUtf8().data());

		return p;
	}
	return NULL;
}
