#include "libreworkfile.h"

struct passwd * LibreworkFile::readPasswdConf( QString name )
{
	Logger::logInfo( QString("readPasswdConf(%1)").arg( name ) );
	struct passwd *conf = NULL;

	if ( ! QFile::exists( LIBREWORK_PASSWD_FILE ) )
	{
		return NULL;
	}

	QFileInfo fileInfo( LIBREWORK_PASSWD_FILE );
	if ( fileInfo.ownerId() != 0 || fileInfo.permission(
		QFileDevice::ReadGroup |
		QFileDevice::WriteGroup |
		QFileDevice::ExeGroup |
		QFileDevice::ReadOther |
		QFileDevice::WriteOther |
		QFileDevice::ExeOther ) )
	{
		Logger::logError( QString("Librework passwd file '%1' owner must be root (id=0), and permissions must be 0600").arg(LIBREWORK_PASSWD_FILE) );
		return NULL;
	}

	FILE *fd;

        if ((fd = fopen(LIBREWORK_PASSWD_FILE, "r")) == NULL )
	{
		Logger::logError( QString("Cannot open passwd file '%1'").arg(LIBREWORK_PASSWD_FILE) );
		return NULL;
	}

	bool found = false;
	while ( ! found )
	{
		conf = fgetpwent(fd);
		if ( conf == NULL )
		{
			found = true;
		} else {
			if ( QString(conf->pw_name) == name )
			{
				found = true;
			}
		}
	}

	fclose(fd);

	return conf;
}

struct spwd * LibreworkFile::readShadowConf( QString name )
{
	Logger::logInfo( QString("readShadowConf(%1)").arg( name ) );
	struct spwd *conf = NULL;

	if ( ! QFile::exists( LIBREWORK_SHADOW_FILE ) )
	{
		return NULL;
	}

	QFileInfo fileInfo( LIBREWORK_SHADOW_FILE );
	if ( fileInfo.ownerId() != 0 || fileInfo.permission(
		QFileDevice::ReadGroup |
		QFileDevice::WriteGroup |
		QFileDevice::ExeGroup |
		QFileDevice::ReadOther |
		QFileDevice::WriteOther |
		QFileDevice::ExeOther ) )
	{
		Logger::logError( QString("Librework shadow file '%1' owner must be root (id=0), and permissions must be 0600").arg(LIBREWORK_SHADOW_FILE) );
		return NULL;
	}

	FILE *fd;

        if ((fd = fopen(LIBREWORK_SHADOW_FILE, "r")) == NULL )
	{
		Logger::logError( QString("Cannot open shadow file '%1'").arg(LIBREWORK_SHADOW_FILE) );
		return NULL;
	}

	bool found = false;
	while ( ! found )
	{
		conf = fgetspent(fd);
		if ( conf == NULL )
		{
			found = true;
		} else {
			if ( QString( conf->sp_namp ) == name )
			{
				found = true;
			}
		}
	}

	fclose(fd);

	return conf;
}

struct group * LibreworkFile::readGroupConf( QString name )
{
	Logger::logInfo( QString("readGroupConf(%1)").arg( name ) );
	struct group *conf = NULL;

	if ( ! QFile::exists( LIBREWORK_GROUP_FILE ) )
	{
		return NULL;
	}

	FILE *fd;

        if ((fd = fopen(LIBREWORK_GROUP_FILE, "r")) == NULL )
	{
		Logger::logError( QString("Cannot open group file '%1'").arg(LIBREWORK_GROUP_FILE) );
		return NULL;
	}

	bool found = false;
	while ( ! found )
	{
		conf = fgetgrent(fd);
		if ( conf == NULL )
		{
			found = true;
		} else {
			if ( QString(conf->gr_name) == name )
			{
				found = true;
			}
		}
	}

	fclose(fd);

	return conf;
}

QString LibreworkFile::getName( uid_t uid )
{
	QString ret = QString::null;

	if ( ! QFile::exists( LIBREWORK_PASSWD_FILE ) )
	{
		return QString::null;
	}

	QFile file( LIBREWORK_PASSWD_FILE );
	if ( ! file.open(QIODevice::ReadOnly | QIODevice::Text) )
	{
		return QString::null;
	}

	while ( ! file.atEnd())
	{
		QByteArray line = file.readLine();
		QString uidString = QString::number( uid );
		QList<QByteArray> lineFields = line.split(':');
		
		if ( lineFields.count() >= 3 )
		{
			if ( lineFields.at(2) == uidString )
			{
				ret = lineFields.at(0);
			}
		}
	}

	return ret;
}

QString LibreworkFile::getGroupName( gid_t gid )
{
	QString ret = QString::null;

	if ( ! QFile::exists( LIBREWORK_GROUP_FILE ) )
	{
		return QString::null;
	}

	QFile file( LIBREWORK_GROUP_FILE );
	if ( ! file.open(QIODevice::ReadOnly | QIODevice::Text) )
	{
		return QString::null;
	}

	while ( ! file.atEnd())
	{
		QByteArray line = file.readLine();
		QString gidString = QString::number( gid );
		QList<QByteArray> lineFields = line.split(':');
		
		if ( lineFields.count() >= 3 )
		{
			if ( lineFields.at(2) == gidString )
			{
				ret = lineFields.at(0);
			}
		}
	}

	return ret;
}

QString LibreworkFile::getLine( int lineNumber )
{
	QString ret = QString::null;

	if ( ! QFile::exists( LIBREWORK_PASSWD_FILE ) )
	{
		return QString::null;
	}

	QFile file( LIBREWORK_PASSWD_FILE );
	if ( ! file.open(QIODevice::ReadOnly | QIODevice::Text) )
	{
		return QString::null;
	}

	QByteArray line;
	for ( int i = 0; i <= lineNumber; i++ )
	{
		if ( ! file.atEnd() )
		{
			line = file.readLine();
		} else {
			line = "";
			i = lineNumber;
		}
	}
	
	if ( ! line.isEmpty() )
	{
		ret = line;
	}

	return ret;
}
